/*
 * log.h
 *
 * Copyright 2013 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef GRUBBY_LOG_H
#define GRUBBY_LOG_H 1

extern int log_time(FILE *log);
extern int log_vmessage(FILE *log, const char *msg, va_list ap);
extern int log_message(FILE *log, const char *msg, ...);

#endif /* GRUBBY_LOG_H */
